	TITLE	I/O DRIVERS
***************************************************************
*     Sample I/O drivers for the CUBIX operating system.      *
*-------------------------------------------------------------*
* The drivers are designed to deal with four 6551 type serial *
* devices, and a 765 type floppy disk controller controlling  *
* up to four standard 40 track single or double sided floppy  *
* diskette drives.                                            *
*-------------------------------------------------------------*
* Although these drivers are fully functional and may be used *
* in a port of the system, their primary purpose is intended  *
* to be as an example of CUBIX to I/O driver interfaceing. As *
* such the device control side of the drivers (Which will be  *
* VERY system specific) has been kept very simple and easy to *
* follow. In particular, no interrupt lines are used, and all *
* I/O operations are accomplished via software polling.       *
*-------------------------------------------------------------*
*             Copyright 1983-2004 Dave Dunfield               *
***************************************************************
*
* CUBIX SYSTEM ADDRESSES
*
DRIVERS	EQU	$E99F		FIRST FREE LOCATION IN ROM
*
* HARDWARE I/O ADDRESSES
*
PIA_A		equ	$F000		; PIA PORT A
CON_A		equ	$F001		; PIA CONTROLL A
PIA_B		equ	$F002		; PIA PORT B
CON_B		equ	$F003		; PIA CONTROLL B
UART1		EQU	$F004		; SERIAL PORT 1 (I/O Card)

IDELO		EQU       $F120			; DATA PORT (LOW BYTE)
IDEERR		EQU       $F121			; READ: ERROR REGISTER; WRITE: PRECOMP
IDESECTC	EQU       $F122			; SECTOR COUNT
IDESECTN	EQU       $F123			; SECTOR NUMBER
IDECYLLO	EQU       $F124			; CYLINDER LOW
IDECYLHI	EQU       $F125			; CYLINDER HIGH
IDEHEAD		EQU       $F126			; DRIVE/HEAD
IDESTTS		EQU       $F127			; READ: STATUS; WRITE: COMMAND
IDEHI		EQU       $F128			; DATA PORT (HIGH BYTE)
IDECTRL		EQU       $F12E			; READ: ALTERNATIVE STATUS; WRITE; DEVICE CONTROL
IDEADDR		EQU       $F12F			; DRIVE ADDRESS (READ ONLY)

FMSR		EQU	$F136		; ADDRESS OF MAIN STATUS REGISTER
FDATA		EQU	$F137		; FLOPPY DATA REGISTER
FLATCH		EQU	$F13A		; FLOPPY CONFIGURATION LATCH

PORTA	EQU 	$F01F
PORTB	EQU 	$F010
PORTC	EQU 	$F02F	

DDRA	EQU 	$F013
DDRB	EQU 	$F012
DDRC	EQU 	$F023

;
; FDC CONFIGURATION LATCH OUTPUT BIT PATTERNS
MOTOR		EQU	%00000000	; BIT PATTERN IN LATCH FOR MOTOR CONTROL (ON)
TERMCN		EQU	%00000001	; BIT PATTERN IN LATCH TO WRITE A TC STROBE
RESETL		EQU	%00000010	; BIT PATTERN IN LATCH TO RESET ALL BITS
MINI		EQU	%00000100	; BIT PATTERN IN LATCH TO SET MINI MODE FDC9229 LOW DENS=1, HIGH DENS=0
PRECOMP		EQU	%00100000	; BIT PATTERN IN LATCH TO SET WRITE PRECOMP 125 NS:
FDDENSITY	EQU	%01000000	; BIT PATTERN IN LATCH TO FLOPPY LOW DENSITY (HIGH IS 0)
FDREADY		EQU	%10000000	; BIT PATTERN IN LATCH TO FLOPPY READY (P-34):


*
* DISK CONTROL BLOCK FORMAT
*
;	ORG	0
;DRIVE	RMB	1		DRIVE ID (ADDRESS)
;NCYL	RMB	1		NUMBER OF CYLINDERS
;NHEAD	RMB	1		NUMBER OF HEADS
;NSEC	RMB	1		NUMBER OF SECTORS/TRACK
;CYL	RMB	1		CURRENT CYLINDER
;HEAD	RMB	1		CURRENT HEAD
;SEC	RMB	1		CURRENT SECTOR
*
;	ORG	DRIVERS
*
* INITIALIZE SYSTEM HARDWARE. ON ENTRY 'Y'
* POINTS TO SYSTEM TABLE TO BE FILLED IN.
* 
* INITIALIZE CUBIX SYSTEM TABLE
HWINIT	LDX	#RITAB		POINT TO OUR TABLE
	LDB	#RISIZ		SIZE OF TABLE
HWIN1	LDA	,X+		GET A BYTE FROM TABLE
	STA	,Y+		WRITE IT TO CUBIX RAM
	DECB			REDUCE COUNT
	BNE	HWIN1		MOVE ENTIRE TABLE
* SOFT RESET THE IDE CHANNEL	
	JSR	IDE_SOFT_RESET
	JSR	SETUPDRIVE
* INITIALIZE THE ECB COMUNICATIONS INTERFACE
	LDAA	#$00			; CLEAR ACCUMULATOR A
	STAA	CON_A			; SET DATA DIRECTION REGISTER A
	STAA	CON_B			; SET DATA DIRECTION REGISTER B
	STAA	PIA_A			; ESTABLISH PA0-PA7 AS INPUTS
	LDAA	#%00101110		; SELECT ORA; SET MODE
	STAA	CON_A			; CONTROL FOR "A" SIDE
	LDAA	#$FF			; ESTABLISH PB0-PB7 AS OUTPUTS
	STAA	PIA_B			;	
	LDAA	#%00101110		; SELECT ORB; SET MODE
	STAA	CON_B			; CONTROL FOR "B" SIDE
	CLRA				; CLEAR ACCUMULATOR A
* INITIALIZE SERIAL PORTS
	LDD	#$0B1E			; 9600, 8 BITS, NO PARITY, 1 STOP BIT
	LDX	#UART1			; POINT TO FIRST UART
	CLR	1,X			; RESET UART
	STD	2,X			; INITIALIZE CONTROL REGISTERS
	LDA	#$0D			; CARRIAGE RETURN
	
* NOTE: HWINIT FALLS INTO WRSER1, CAUSING ITS LAST OPERATION
*       TO BE AN OUTPUT TO THIS DEVICE (THE CONSOLE).
*
* SERIAL PORT WRITE DRIVERS
*
WRSER1	PSHS	A			; STORE A
	LDAA	#$02			; FUNCTION 02, OUT CHAR
	JSR	ECB_OUTCHAR		; SEND FUNCTION REQUEST
	PULS	A			; RESTORE A
	JSR	ECB_OUTCHAR		; FUNCTION REQUIRES ONE BYTE PARAMETER	
	RTS
*
* SERIAL PORT WRITE DRIVERS
*
WRSER2	LDX	#UART1			; ADDRESS OF UART 1
* WRITE CHARACTER(A) TO UART
WRSER	LDB	1,X			; GET STATUS
	BITB	#%00010000		; IS TX READY
	BEQ	WRSER			; NO, WAIT FOR IT
	STA	,X			; WRITE DATA
	RTS

*
* SERIAL PORT READ DRIVERS
*
RDSER1	LDAA	#$01			; FUNCTION 01, GET KB IN
	JSR	ECB_OUTCHAR		; SEND FUNCTION REQUEST
	JSR	ECB_INCHAR		; FUNCTION RETURNS ONE BYTE
	BCC	IOF_CONIN_SUCCESS	;
	JMP	IOF_CONIN_NULL		; TIMED OUT
IOF_CONIN_SUCCESS:			;	
	CMPA	#$00			;	
	BEQ	IOF_CONIN_NULL		;
	CLRB
	RTS
IOF_CONIN_NULL:				;	
	LDAA	#$FF			; TIMED OUT
;	CLRB				;
	RTS				;
RDSER2	LDX	#UART1			; ADDRESS OF UART 1
* READ CHARACTER FROM UART
	LDA	1,X			; GET STATUS REGISTER
	BITA	#%00001000		; IS RX READY
	BEQ	RDNULL			; NO, INDICATE NO CHAR
	LDA	,X			; GET DATA CHAR
	ORCC	#%00000100		; SET 'Z'
	RTS

* NULL DEVICE DRIVERS
RDNULL	LDA	#$FF		INDICATE NO CHARACTER
WRNULL	RTS			IGNORE OPERATION
*
* FORMAT DISK ('U' POINTS TO DCB), INTERLEAVE FACTOR IN 'A'
*
DFORMAT
	LDAA	DRIVE,U			; GET DRIVE
	CMPA	#$01			; DRIVE B?
	BNE 	NOTFDB			;
	JMP	FORMFL			;
NOTFDB:	
	RTS
*
* HOME HEAD ON DRIVE ('U' POINTS TO DCB)
*
DHOME
	LDAA	DRIVE,U			; GET DRIVE
	CMPA	#$01			; DRIVE A?
	BNE 	NOTHDB			;
	LDAA	#$00	
	JMP	SETTRACK		;
NOTHDB:
	LDA	#$03			; HOME DISK
	JSR	ECB_OUTCHAR		;
	LDA	DRIVE,U			; GET DRIVE
	JSR	ECB_ENC_OUTCHAR		; SEND TO ECB

	RTS
*
* SEEK TO CYLINDER ON DRIVE ('U' POINTS TO DCB)
*
DSEEK	
	LDAA	DRIVE,U			; GET DRIVE
	CMPA	#$01			; DRIVE A?
	BNE 	NOTSDB			;
	JMP	SETTRACK		;
NOTSDB:
	RTS	
*
* READ A SECTOR, FROM DISK ('U' POINTS TO DCB) TO MEMORY(X)
*
DRDSEC
	LDAA	DRIVE,U			; GET DRIVE
	CMPA	#$00			; DRIVE A?
	BNE 	NOTRDA			;
	JMP	Z80RDRIVE		;
NOTRDA	
	CMPA	#$01			; DRIVE B?
	BNE 	NOTRDB			;
	JMP	READFL			;
NOTRDB
	CMPA	#$02			; DRIVE C?
	BNE 	NOTRDC			;
	JMP	Z80RDRIVE		;
NOTRDC
	CMPA	#$03			; DRIVE D?
	BNE 	NOTRDD			;
	JMP	IDE_READ_SECTOR		;
NOTRDD
	RTS	
	
Z80RDRIVE	
	LDAA	#$04
	JSR	ECB_OUTCHAR		; DO READ SECTOR
	LDAA	DRIVE,U			; GET DRIVE
	JSR	ECB_ENC_OUTCHAR		; SEND TO ECB
	LDAA	HEAD,U			; GET CURRENT HEAD
	JSR	ECB_ENC_OUTCHAR		; SEND TO ECB
	LDAA	CYL,U			; GET CYLINDER ID
	JSR	ECB_ENC_OUTCHAR		; SEND TO ECB
	LDAA	SEC,U			; GET CYLINDER ID
	JSR	ECB_ENC_OUTCHAR		; SEND TO ECB
	JSR	ECB_DEC_CHARIN		; GET STATUS BYTE
	CMPA	#$FF			; IS ERROR?
	BEQ	DRDSECA			; YES, ABORT	
	LDAB	#$0000			; 
DRDSEC1	
	LDAA	#$06			; GET DISK BUFFER BYTE
	JSR	ECB_OUTCHAR		;
	LDAA	#$00			;
	JSR	ECB_ENC_OUTCHAR		;
	PSHS	B
	PULS	A			;
	JSR	ECB_ENC_OUTCHAR		;
	JSR	ECB_DEC_CHARIN		; GET NEXT BYTE
	STAA	,X+			; WRITE IT TO CUBIX RAM
	INCB				; BUMP COUNT
	BNE	DRDSEC1			; DONE? NO, LOOP
	LDAB	#$0000			; 
DRDSEC2
	LDAA	#$06			; GET DISK BUFFER BYTE
	JSR	ECB_OUTCHAR		;
	LDAA	#$01			;
	JSR	ECB_ENC_OUTCHAR		;
	PSHS	B
	PULS	A			;
	JSR	ECB_ENC_OUTCHAR		;
	JSR	ECB_DEC_CHARIN		; GET NEXT BYTE
	STA	,X+			; WRITE IT TO CUBIX RAM
	INCB				; BUMP COUNT
	BNE	DRDSEC2			; DONE? NO, LOOP
	LDAA	#00			; SET NO ERROR
	RTS
DRDSECA	LDAA	#02			; RETURN ERROR CODE	
	RTS

*
* WRITE A SECTOR TO DISK ('U' POINTS TO DCB) FROM MEMORY(X)
*
DWRSEC	
	LDAA	DRIVE,U			; GET DRIVE
	CMPA	#$00			; DRIVE A?
	BNE 	NOTWDA			;
	JMP	Z80WDRIVE		;
NOTWDA	
	CMPA	#$01			; DRIVE B?
	BNE 	NOTWDB			;
	JMP	WRITEFL			;
NOTWDB
	CMPA	#$02			; DRIVE C?
	BNE 	NOTWDC			;
	JMP	Z80WDRIVE		;
NOTWDC
	CMPA	#$03			; DRIVE D?
	BNE 	NOTWDD			;
	JMP	IDE_WRITE_SECTOR	;
NOTWDD
	RTS		
Z80WDRIVE
	LDAB	#$0000			; 
DWRSEC1	
	LDAA	#$07			; GET DISK BUFFER BYTE
	JSR	ECB_OUTCHAR		;
	LDAA	#$00			;
	JSR	ECB_ENC_OUTCHAR		;
	PSHS	B
	PULS	A			;
	JSR	ECB_ENC_OUTCHAR		;
	LDAA	,X+			; GET NEXT BYTE
	JSR	ECB_ENC_OUTCHAR		; SEND IT
	INCB				; BUMP COUNT
	BNE	DWRSEC1			; DONE? NO, LOOP
	LDAB	#$0000			; 
DWRSEC2	
	LDAA	#$07			; GET DISK BUFFER BYTE
	JSR	ECB_OUTCHAR		;
	LDAA	#$01			;
	JSR	ECB_ENC_OUTCHAR		;
	PSHS	B
	PULS	A			;
	JSR	ECB_ENC_OUTCHAR		;
	LDAA	,X+			; GET NEXT BYTE
	JSR	ECB_ENC_OUTCHAR		; SEND IT
	INCB				; BUMP COUNT
	BNE	DWRSEC2			; DONE? NO, LOOP


	LDAA	#$05
	JSR	ECB_OUTCHAR		; DO READ SECTOR
	LDAA	DRIVE,U			; GET DRIVE
	JSR	ECB_ENC_OUTCHAR		; SEND TO ECB
	LDAA	HEAD,U			; GET CURRENT HEAD
	JSR	ECB_ENC_OUTCHAR		; SEND TO ECB
	LDAA	CYL,U			; GET CYLINDER ID
	JSR	ECB_ENC_OUTCHAR		; SEND TO ECB
	LDAA	SEC,U			; GET CYLINDER ID
	JSR	ECB_ENC_OUTCHAR		; SEND TO ECB
	LDAB	$00			; SIZE OF SECTOR/2	

	JSR	ECB_DEC_CHARIN		; GET STATUS BYTE
	CMPA	#00			; IS ERROR?
	BNE	DRDSECA			; YES, ABORT
	LDAA	#00			; SET NO ERROR
	RTS
DWRSECA	LDAA	#02			; RETURN ERROR CODE		
	RTS
	
	
*
* UTILITY FUNCTIONS
*
	
ECB_INCHAR:				;
	PSHS	X			;
	LDX	#$FFFF			;
ECB_INCHAR_LOOP:			;
	DEX				;
	CMPX	#$0000			;
	BEQ	ECB_INCHAR_ABORT	;
	TST	CON_A			; IS BYTE READY (CA1 TRANSITION)
	BPL	ECB_INCHAR_LOOP		; NO, TRY AGAIN
	LDAA	PIA_A			; READ FROM PORT A WITH CA2 STROBES
	PULS	X			;
	CLC				; SIGNAL SUCCESS
	RTS				;
ECB_INCHAR_ABORT:			;
	PULS	X			;
	SEC				; SIGNAL FAILURE
	RTS				;

ECB_OUTCHAR:				;
	PSHS	B			; SAVE B-REG (COUNTER)
	STAA	PIA_B			; WRITE TO PORT B WITH CB2 STROBES
	LDAA	PIA_B			; RESET BIT 7 CRB
ECB_OUTCHAR_LOOP:			;
	TST	CON_B			; IS BYTE READY (CB1 TRANSITION)
	BPL	ECB_OUTCHAR_LOOP	; NO, TRY AGAIN
	LDAB	PIA_B			; RESET BIT 7 CRA WITH OUTPUT REGISTER READ
	PULS	B			; RESTORE B-REG
	RTS

ECB_ENC_OUTCHAR:			;
	CMPA	#27			; IS 27?
	BEQ	ECB_ENC_OUTCHAR_27	;
	CMPA	#00			; IS NULL?
	BEQ	ECB_ENC_OUTCHAR_00	;
	BRA	ECB_OUTCHAR		;
ECB_ENC_OUTCHAR_27:			;
	JSR	ECB_OUTCHAR		;
	LDAA	#27			;
	BRA	ECB_OUTCHAR		;
ECB_ENC_OUTCHAR_00:			;
	LDAA	#27			;
	JSR	ECB_OUTCHAR		;
	LDAA	#'0'			;
	BRA	ECB_OUTCHAR		;

	

;__ECB_DEC_CHARIN_____________________________________________________________________________________ 
;
;	GET A DECODED BYTE FROM THE M6809
;	A= RETURNED BYTE
;
;	THIS WILL RETURN ALL BYTES
;_____________________________________________________________________________________________________
;
ECB_DEC_CHARIN:
	JSR	ECB_INCHAR		; GET BYTE
	BCS	ECB_DEC_CHARIN		;
	RTS				;

		
*__IDE_READ_SECTOR___________________________________________________________________________________
*
*  READ IDE SECTOR (IN LBA) INTO BUFFER
*     
*____________________________________________________________________________________________________				
IDE_READ_SECTOR:		
		PSHS	X			; STORE BUFFER LOCATION
		JSR	IDE_WAIT_BUSY_READY	; MAKE SURE DRIVE IS READY TO PROCEED
		BNE	IDE_READ_SECTOR_ERR	; ERROR, ABORT
		JSR	IDE_SETUP_LBA		; TELL DRIVE WHAT SECTOR IS REQUIRED
		LDAA    #$20			;  		
		STAA    IDESTTS			; $20 = IDE 'READ SECTOR' COMMAND 		
		JSR	IDE_WAIT_BUSY_READY	; MAKE SURE DRIVE IS READY TO PROCEED
		BNE	IDE_READ_SECTOR_ERR	; ERROR, ABORT
		JSR	IDE_TEST_ERROR		; ENSURE NO ERROR WAS REPORTED
		BNE	IDE_READ_SECTOR_ERR	; ERROR, ABORT
		JSR	IDE_WAIT_BUFFER		; WAIT FOR FULL BUFFER SIGNAL FROM DRIVE
		BNE	IDE_READ_SECTOR_ERR	; ERROR, ABORT
		PULS	X			; GET BUFFER LOCATION
		JSR	IDE_READ_BUFFER		; GRAB THE 256 WORDS FROM THE BUFFER
		CLRA				; ZERO = 1 ON RETURN = OPERATION OK
		RTS
IDE_READ_SECTOR_ERR:
		LDAA    #$02
		RTS
		
		
*__IDE_WRITE_SECTOR__________________________________________________________________________________
*
*  WRITE IDE SECTOR (IN LBA) FROM BUFFER
*     
*____________________________________________________________________________________________________				
IDE_WRITE_SECTOR:
		PSHS 	X			; STORE BUFFER LOCATION
		JSR	IDE_WAIT_BUSY_READY	; MAKE SURE DRIVE IS READY TO PROCEED
		BNE	IDE_WRITE_SECTOR_ERR	; ERROR, ABORT
		JSR	IDE_SETUP_LBA		; TELL DRIVE WHAT SECTOR IS REQUIRED
		LDAA    #$30			;  
		STAA    IDESTTS			; $30 = IDE 'WRITE SECTOR' COMMAND 
		JSR	IDE_WAIT_BUSY_READY	;
		BNE	IDE_WRITE_SECTOR_ERR	; ERROR, ABORT
		JSR	IDE_TEST_ERROR		; ENSURE NO ERROR WAS REPORTED
		BNE	IDE_WRITE_SECTOR_ERR	; ERROR, ABORT
		JSR	IDE_WAIT_BUFFER		; WAIT FOR BUFFER READY SIGNAL FROM DRIVE
		BNE	IDE_WRITE_SECTOR_ERR	; ERROR, ABORT
		PULS	X			; GET BUFFER LOCATION
		JSR	IDE_WRITE_BUFFER       	; SEND 256 WORDS TO DRIVE'S BUFFER
		JSR	IDE_WAIT_BUSY_READY	; MAKE SURE DRIVE IS READY TO PROCEED
		BNE	IDE_WRITE_SECTOR_ERR	; ERROR, ABORT
		JSR	IDE_TEST_ERROR		; ENSURE NO ERROR WAS REPORTED
		BNE	IDE_WRITE_SECTOR_ERR	; ERROR, ABORT
		CLRA				; ZERO = 1 ON RETURN = OPERATION OK		
		RTS
IDE_WRITE_SECTOR_ERR:
		LDA	#$02
		RTS
*__IDE_SOFT_RESET____________________________________________________________________________________
*
*  SOFT RESET IDE CHANNEL
*     
*____________________________________________________________________________________________________				
IDE_SOFT_RESET

		LDAA    #%00000110		; NO INTERRUPTS, RESET DRIVE = 1
		STAA    IDECTRL			;
		LDAA    #%00000010		; NO INTERRUPTS, RESET DRIVE = 0
		STAA    IDECTRL			;
		JSR	IDE_WAIT_BUSY_READY	;
		RTS				;


*__IDE_WAIT_BUSY_READY_______________________________________________________________________________
*
*  WAIT FOR IDE CHANNEL TO BECOME READY
*     
*____________________________________________________________________________________________________				
IDE_WAIT_BUSY_READY:
		
		LDX	#$0000			;
IDE_WBSY:
		INX				;
		CPX	#$1E00			; TIMED OUT?
		BEQ	IDE_TO			; YUP, EXIT
		LDAA    IDESTTS			; READ ERROR REG
		ANDA    #%11000000		; MASK OFF BUSY AND RDY BITS
		CMPA   	#%01000000		; WE WANT BUSY(7) TO BE 0 AND RDY(6) TO BE 1
		BNE	IDE_WBSY		;
		CLRA				; ZERO 1 = OK
		RTS
IDE_TO:
		LDAA    #$FF			; ZERO 0 = TIMED OUT
		CLRA
		RTS
		
*__IDE_TEST_ERROR____________________________________________________________________________________
*
*  TEST FOR IDE ERROR
*     
*____________________________________________________________________________________________________				
IDE_TEST_ERROR:
		
		LDAA    IDESTTS			;
		ANDA    #%00000001		;
		BNE	IDE_ERR			;		
		CLRA				;
		RTS
IDE_ERR:
		LDAA    IDEERR			; READ ERROR FLAGS
		LDAA    #$FF			; ZERO 0 = ERROR
		RTS				;
						;
*__IDE_WAIT_BUFFER____________________________________________________________________________________
*
*  WAIT FOR IDE BUFFER TO FILL
*     
*____________________________________________________________________________________________________				
IDE_WAIT_BUFFER:
		LDX	#$0000			;
IDE_WDRQ:
		INX				;
		CPX	#$FFFF			;
		BEQ	IDE_TO2			;
		LDAA    IDESTTS			; WAIT FOR DRIVE'S 512 BYTE READ BUFFER 
		ANDA    #%00001000		;
		BNE	IDE_WDRQ		;
		CLRA				; ZERO 1 = OK
		RTS
IDE_TO2:
		LDAA    #$FF			; CARRY 0 = TIMED OUT
		CLRA
		RTS
*__IDE_READ_BUFFER___________________________________________________________________________________
*
*  READ IDE BUFFER LITTLE ENDIAN
*     
*____________________________________________________________________________________________________				
IDE_READ_BUFFER:
		LDY    	#$0100			; 256 WORDS (512 BYTES PER SECTOR)
IDEBUFRD:
		LDAA    IDELO			; LOW BYTE OF WORD FIRST
		LDAB    IDEHI			; THEN HIGH BYTE OF WORD
		STAA    ,X+			; 'ID DRIVE' IDE RESPONSE IS LITTLE ENDIAN FORMAT		
		STAB    ,X+			; 'ID DRIVE' IDE RESPONSE IS LITTLE ENDIAN FORMAT
		DEY     			;
		CMPY    #$0000			;
		BNE	IDEBUFRD		;
		RTS				;
*__IDE_WRITE_BUFFER___________________________________________________________________________________
*
*  WRITE IDE BUFFER LITTLE ENDIAN
*     
*____________________________________________________________________________________________________				
IDE_WRITE_BUFFER:
		LDAB    #$00			; 256 WORDS (512 BYTES PER SECTOR)
IDEBUFWT:
		LDAA    1,X			; SECTORS ARE BIG ENDIAN
		STAA    IDEHI			; SET UP HIGH LATCHED BYTE BEFORE
		LDAA    ,X++			; SECTORS ARE BIG ENDIAN
		STAA    IDELO			; WRITING WORD WITH WRITE TO LOW BYTE
		DECB     			;
		CMPB    #$00			;
		BNE	IDEBUFWT		;
		RTS				;
		
*__IDE_SETUP_LBA_____________________________________________________________________________________
*
*  SETUP LBA DATA
*     
*____________________________________________________________________________________________________				
IDE_SETUP_LBA:	   
		LDAA    #$01			;  
		STAA    IDESECTC		; SET SECTOR COUNT = 1
		LDAA    SEC,U			;
		STAA    IDESECTN		; SET LBA 0:7
		LDAA    CYL,U			;
		STAA    IDECYLLO		; SET LBA 8:15
		LDAA    HEAD,U			;
		STAA    IDECYLHI		; SET LBA 16:23
		LDAA    #$01			;
		ANDA    #%00001111		; LOWEST 4 BITS USED ONLY
		ORA     #%11100000		; TO ENABLE LBA MODE
		STAA    IDEHEAD			; SET LBA 24:27 + BITS 5:7=111
		JSR	SEGDISPLAY		;
		RTS

;__SETUPDRIVE__________________________________________________________________________________________________________________________ 
;
;	SETUP FLOPPY DRIVE SETTINGS 
;________________________________________________________________________________________________________________________________
;
;
;
SETUPDRIVE:
	LDAA	#RESETL			; RESET SETTINGS
	ORAA	#MINI			; SELECT MINI FLOPPY (low dens=1, high dens=0)
	ORAA	#PRECOMP		; SELECT PRECOMP 
	ORAA	#FDDENSITY		; SELECT DENSITY
	ORAA	#FDREADY		; SELECT READY SIGNAL
	STAA	FLATCH_STORE		; SAVE SETTINGS
	JSR	CHECKINT		;
	LDA	#$03			; SPECIFY COMMAND
	JSR	PFDATA			; OUTPUT TO FDC
	LDA	#$7F			; 6 MS STEP, 480 MS HEAD UNLOAD
	JSR	PFDATA			; OUTPUT TO FDC
	LDA	#$05			; 508 MS HEAD LOAD, NON-DMA MODE
	JSR	PFDATA			; OUTPUT TO FDC
	RTS

;__OUTFLATCH__________________________________________________________________________________________________________________________ 
;
;	SEND SETTINGS TO FLOPPY CONTROLLER
;________________________________________________________________________________________________________________________________
;
OUTFLATCH:
	LDAA	FLATCH_STORE		; SET A TO SETTINGS
	STAA	FLATCH			; OUTPUT TO CONTROLLER
	RTS

	
;__FORMFL________________________________________________________________________________________________________________________ 
;
; 	FORMFL 	
;________________________________________________________________________________________________________________________________
;	
FORMFL:	
	CLR	TCYL		START WITH CYLINDER 0
* FORMAT ONE CYLINDER
FORCYL	LDAA	FLATCH_STORE		; POINT TO FLATCH
	ANDA    #%11111101		;
	STAA	FLATCH_STORE		; POINT TO FLATCH
	JSR	OUTFLATCH		; OUTPUT TO CONTROLLER	
	JSR	CHECKINT		; CHECK INTERRUPT STATUS, MAKE SURE IT IS CLEAR
	JSR	SETTRACK	MOVE TO THE CYLINDER
	CLR	THEAD		START WITH BOTTOM HEAD
	
	
* FORMAT ONE HEAD
FORHED	LDB	#$00		SET SECTOR ID
	LDA	#$4D		FORMAT COMMAND
	JSR	PFDATA		OUTPUT TO FDC
	LDA	THEAD		GET CURRENT HEAD
	ASLA			SHIFT INTO...
	ASLA			PROPER POSITION
	ORA	DRIVE,U		INCLUDE DRIVE
	JSR	PFDATA		OUTPUT TO FDC
	LDA	#$02		512 BYTES/SECTOR
	JSR	PFDATA		OUTPUT TO FDC
	LDA	NSEC,U		SECTORS/TRACK
	JSR	PFDATA		OUTPUT TO FDC
	LDA	#$2A		GAP LENGTH ($38 FOR 10 S/TRK)
	JSR	PFDATA		OUTPUT TO FDC
	LDA	#$E5		FILLER BYTE
	JSR	PFDATA		OUTPUT TO FDC
* FORMAT ONE SECTOR
FORSEC	LDAA	>FMSR		READ FDC STATUS
	BPL	FORSEC		WAIT TILL READY FOR DATA
	BITA	#%00100000		; 
	BEQ	DSKFMTEND		;
	LDAA	TCYL		GET CYLINDER ID
	STAA	>FDATA		WRITE TRACK TO FDC
FORS2	LDAA	>FMSR		READ FDC STATUS
	BPL	FORS2		WAIT TILL READY FOR DATA
	BITA	#%00100000		; 
	BEQ	DSKFMTEND		;
	LDAA	THEAD		GET HEAD ID
	STAA	>FDATA		WRITE HEAD TO FDC
FORS3	LDAA	>FMSR		READ FDC STATUS
	BPL	FORS3		WAIT TILL READY FOR DATA
	BITA	#%00100000		; 
	BEQ	DSKFMTEND		;
	INCB			ADD 1
	STAB	>FDATA		WRITE SECTOR TO FDC
FORS5	LDAA	>FMSR		READ FDC STATUS
	BPL	FORS5		WAIT TILL READY FOR DATA
	BITA	#%00100000		; 
	BEQ	DSKFMTEND		;
	LDAA	#$02		512 BYTES/SECTOR
	STAA	>FDATA		WRITE DATA BYTE	
	CMPB	#$09		OVER?
	BNE	FORSEC		NO, ITS OK
	JSR	DSKOPEND	EVERYTHING OK?
	INC	THEAD		MOVE TO NEXT HEAD
	LDAA	THEAD		GET VALUE
	CMPA	NHEAD,U		HAVE WE DONE EM ALL?
	BLO	FORHED		NO, DO NEXT HEAD
	INC	TCYL		MOVE TO NEXT CYLINDER
	LDAA	TCYL		GET VALUE
	STAA	CYL,U
	CMPA	NCYL,U		HAVE WE DONE EM ALL?
	BLO	FORCYL1		NO, DO NEXT CYLINDER	
	LDAA	FLATCH_STORE		; POINT TO FLATCH
	ORAA    #%00000010		;
	STAA	FLATCH_STORE		; POINT TO FLATCH
	JSR	OUTFLATCH		; OUTPUT TO CONTROLLER					;

	RTS
DSKFMTEND:
	JMP	DSKOPEND	EVERYTHING OK?
TCYL:	FCB	00
THEAD:	FCB	00
FORCYL1:
	JMP	FORCYL	

;__READFL________________________________________________________________________________________________________________________ 
;
; 	READ A FLOPPY SECTOR 	
;________________________________________________________________________________________________________________________________
;	
READFL:
	JSR	SEGDISPLAY
	LDAA	#$46			; BIT 6 SETS MFM, 06H IS READ COMMAND
	STAA	FCMD
	JMP	DSKOP

;__WRITEFL________________________________________________________________________________________________________________________ 
;
; 	WRITE A FLOPPY SECTOR 	
;________________________________________________________________________________________________________________________________
;	
WRITEFL:
	JSR	SEGDISPLAY
	LDAA	#$45			; BIT 6 SETS MFM, 05H IS WRITE COMMAND
	STAA	FCMD
	JMP	DSKOP
;__DSKOP__________________________________________________________________________________________________________________________ 
;
; 	PERFORM A DISK OPERATION 	
;________________________________________________________________________________________________________________________________
;		
DSKOP:
	JSR	CHECKINT		; CHECK INTERRUPT STATUS, MAKE SURE IT IS CLEAR
	CMPA	#$FF			; DID IT RTSURN WITH ERROR CODE?
	BEQ	DSKEXIT			; IF YES, EXIT WITH ERROR CODE
					;	
 					;
	LDAA	FLATCH_STORE		; POINT TO FLATCH
	ANDA	#%11111101		; SET MOTOR ON
	STAA	FLATCH_STORE		; POINT TO FLATCH	
	JSR	OUTFLATCH		; OUTPUT TO CONTROLLER
					;
	JSR	SETTRACK		; PERFORM SEEK TO TRACK
					;
	LDAA	FCMD			; WHAT COMMAND IS PENDING?
	CMPA	#$46			; IS IT A READ COMMAND?
	BEQ	RDCMD			;
	JMP	WRR_POLL		; NO, MUST BE READ OR WRITE COMMAND
RDCMD:	
	JMP	RDD_POLL		;
DSKEXIT:	
	LDAA	FLATCH_STORE		; POINT TO FLATCH
	ORAA	#%00000010		; SET MOTOR OFF
	STAA	FLATCH_STORE		; POINT TO FLATCH	
	JSR	OUTFLATCH		; OUTPUT TO CONTROLLER
	LDAA	#$FF			; SET -1 IF ERROR
	RTS
SNDFDWR:
	LDY	#512		BYTES/SECTOR COUNT
	LDAA	DRIVE,U			; GET DISK UNIT NUMBER
	ANDA	#$03			; MASK FOR FOUR DRIVES.
	STAA	UNIT			; PARK IT IN TEMP
	LDAA	HEAD,U			; GET HEAD SELECTION
	ANDA	#$01			; INSURE SINGLE BIT
	ASLA				;
	ASLA				; MOVE HEAD TO BIT 2 POSITION
	ORAA	UNIT			; OR HEAD TO UNIT BYTE IN COMMAND BLOCK
	STAA	UNIT			; STORE IN UNIT

	LDAA	FCMD			;
	JSR	PFDATA			; PUSH COMMAND TO I8272
	LDAA	UNIT			;
	JSR	PFDATA			; 
	LDAA	CYL,U			;
	JSR	PFDATA			; 
	LDAA	HEAD,U			;
	JSR	PFDATA			; 
	LDAA	SEC,U			;
	INCA
	JSR	PFDATA			; 
	LDAA	#$02			;
	JSR	PFDATA			; WHAT DENSITY
	LDAA	#$09			;
	JSR	PFDATA			; ASSUME SC (SECTOR COUNT)  EOT
	LDAA	#$0D			;
	JSR	PFDATA			; WHAT GAP IS NEEDED
	LDAA	#$FF			; DTL, IS THE LAST COMMAND BYTE TO I8272
	JSR	PFDATA
	RTS
UNIT	FCB	$00			;
; PERFORM READ
; FROM READ TO READ MUST NOT EXCEED 25US WORST CASE MIN.
; 1 Mhz 6809 = 1 CYC PER US OR 25 CYCLES WORST CASE (50 FOR 2 Mhz)
;	
RDD_POLL:
	JSR	SNDFDWR
RDS1	LDA	>FMSR		GET STATUS
	BPL	RDS1		NOT READY
	BITA	#%00100000	EXECUTION MODE?
	BEQ	DSKOPEND	NO, ERROR
	LDA	>FDATA		GET DATA
	STA	,X+		WRITE IT
	LEAY	-1,Y		REDUCE COUNT
	BNE	RDS1		KEEP GOING
DSKOPEND:
	LDAA	FLATCH_STORE		; POINT TO FLATCH
	ORAA	#%00000001		;
	STAA	FLATCH_STORE		; SET TC
	JSR	OUTFLATCH		; OUTPUT TO CONTROLLER
	NOP				;
	NOP				; 2 MICROSECOND DELAY
	NOP				;
	NOP				; 
	ANDA	#%11111110		;
	STAA	FLATCH_STORE		; CLEAR TC
	JSR	OUTFLATCH		; OUTPUT TO CONTROLLER
	NOP				;
	NOP				; 2 MICROSECOND DELAY
*	NOP				;
*	NOP				; 2 MICROSECOND DELAY
	ORAA    #%00000010		; MOTOR OFF
	STAA	FLATCH_STORE		; POINT TO FLATCH
	JSR	OUTFLATCH		; OUTPUT TO CONTROLLER					;
					;
RSTEXIT:
	CLR	,-S		ZERO RESULT CODE
	JSR	GFDATA		GET ERROR TYPE
	TFR	A,B		'B' = FDC RESULTS
	JSR	GFDATA		READ STATUS REG#2
	BEQ	RESUL4		NO SECOND REGISTER
* ERROR CODES DETECTED, TRANSLATE
	LDY	#RESTAB		GET TABLE
RESUL1	LSLA			SHIFT OUT
	BCS	RESUL2		FOUND ERROR
	BEQ	RESUL3		NO MORE LEFT
	LEAY	1,Y		ADVANCE 'Y'
	BRA	RESUL1		CONTINUE
RESUL2	LDA	,Y		GET RESULT CODE
	STA	,S		SET RESULT CODE
* CLEAR OUT ANY REMAINING DATA
RESUL3	
	JSR	GFDATA		READ BYTE FROM FDC
	BNE	RESUL3		CLEAR THEM ALL
RESUL4	BITB	#%11000000	TEST FOR ANY ERRORS	
	PULS	A,PC

WRR_POLL:
	JSR	SNDFDWR
WRS1	LDB	,X+		GET DATA
WRS2	LDA	>FMSR		GET STATUS
	BPL	WRS2		NOT READY
	BITA	#%00100000	EXECUTION MODE?
	BEQ	WRS3		NO, ERROR
	STB	>FDATA		WRITE TO FDC
	LEAY	-1,Y		BACKUP COUNT
	BNE	WRS1		DO NEXT
WRS3	
	JMP	DSKOPEND		; 

	
		
;__SETTRACK__________________________________________________________________________________________________________________________ 
;
; 	SEEK TO A TRACK ON GIVEN UNIT
; 	A: TRACK #
;________________________________________________________________________________________________________________________________
;
SETTRACK:
	PSHS	A			; STORE A
					; ANY INTERUPT PENDING
					; IF YES FIND OUT WHY/CLEAR
	JSR	CHECKINT		; CHECK INTERRUPT STATUS, MAKE SURE IT IS CLEAR
	CMPA	#$FF			; DID IT RTSURN WITH ERROR CODE?
	BNE	SETTRK1
	JMP	SETTRKEXIT		;
					;
SETTRK1:					
	LDAA	CYL,U			; GET TRACK
	CMPA	#$00			;
	BEQ	RECAL			; IF 0 PERFORM RECAL INSTEAD OF SEEK
	LDAA	#$0F			; SEEK COMMAND
	JSR	PFDATA			; PUSH COMMAND
	LDAA	DRIVE,U			; SAY WHICH UNIT
	JSR	PFDATA			; SEND THAT
	LDAA	CYL,U			; TO WHAT TRACK
	JSR	PFDATA			; SEND THAT TOO
	JMP	WAINT			; WAIT FOR INTERRUPT SAYING DONE
RECAL:
	LDAA	#$07			; RECAL TO TRACK 0
	JSR	PFDATA			; SEND IT
	LDAA	DRIVE,U			; WHICH UNIT
	JSR	PFDATA			; SEND THAT TOO
;
WAINT:
;
	JSR	SEGDISPLAY
SETTRK2:		
	JSR	CHECKINT			
	LDAA	>FMSR			; READ SEEK STATUS
	BITA	#%00001111		; ANY DRIVES SEEKING?
	BNE	SETTRK2			; YES, WAIT FOR THEM
;
SETTRKEXIT:
	PULS	A
	RTS

;__PFDATA__________________________________________________________________________________________________________________________ 
;
; WRITE A COMMAND OR PARAMETER SEQUENCE
;
; TRANSFERS ARE SYNCHONIZED BY MSR D7 <RQM> AND D6 <DIO>
;	RQM  DIO
;	0	0	BUSY
;	1	0	WRITE TO DATA REGISTER PERMITTED
;	1	1	BYTE FOR READ BY HOST PENDING
;	0	1	BUSY
;
;________________________________________________________________________________________________________________________________
;
PFDATA:
	PSHS	A			; SAVE DATA BYTE
WRF1:
	LDAA	>FMSR			; READ FDC STATUS
	BPL	WRF1			; FDC IS NOT READY, WAIT FOR IT
	BITA	#%01000000		; TEST DIO BIT
	BNE	WRF2			; FDC IS OUT OF SYNC
	PULS	A			; RESTORE DATA
	STAA	>FDATA			; WRITE TO FDC
	RTS
*; FDC IS OUT OF SYNC CLEAR IT OUT AND RE-TRY
WRF2:
	LDAA	>FDATA			; READ DATA REGISTER
	BRA	WRF1			; AND CONTINUE

	
;__CHECKINT__________________________________________________________________________________________________________________________ 
;
; CHECK FOR ACTIVE FDC INTERRUPTS BEFORE GIVING I8272 COMMANDS
; POLL RQM FOR WHEN NOT BUSY AND THEN SEND FDC
; SENSE INTERRUPT COMMAND.  IF IT RTSURNS WITH NON ZERO
; ERROR CODE, PASS BACK TO JSRING ROUTINE FOR HANDLING
;________________________________________________________________________________________________________________________________
;
CHECKINT:
	LDAA	>FMSR			; READING OR WRITING IS KEYS TO D7 RQM
	BPL	CHECKINT		; WAIT FOR RQM TO BE TRUE. WAIT UNTIL DONE
	BITA	#%01000000		; WAITING FOR INPUT?
	BNE	ERRCLR
	JMP	SENDINT			; SENSE INTERRUPT COMMAND
ERRCLR:	
	LDAA	>FDATA			; CLEAR THE JUNK OUT OF DATA REGISTER
	LDAA	>FMSR			; CHECK WITH RQM
	ANDA	#$80			; IF STILL NOT READY, READ OUT MORE JUNK
	BEQ	ERRCLR			;
	LDAA	#$FF			; RETURN ERROR CODE -1
					;
	RTS

;__SENDINT__________________________________________________________________________________________________________________________ 
;
; SENSE INTERRUPT COMMAND
;________________________________________________________________________________________________________________________________
;			
SENDINT:
	LDAA	#$08			; SENSE INTERRUPT COMMAND
	JSR	PFDATA			; SEND IT
	JSR	GFDATA			; GET RESULTS
	STAA	ST0			; STORE THAT
	ANDA	#$C0			; MASK OFF INTERRUPT STATUS BITS
	CMPA	#$80			; CHECK IF INVALID COMMAND
	BEQ	ENDSENDINT		; YES, EXIT
	JSR	GFDATA			; GET ANOTHER (STATUS CODE 1)
	LDAA	ST0			; GET FIRST ONE
	ANDA	#$C0			; MASK OFF ALL BUT INTERRUPT CODE 00 IS NORMAL
ENDSENDINT:
	RTS				;ANYTHING ELSE IS AN ERROR

	
;__GFDATA__________________________________________________________________________________________________________________________ 
;
; GET DATA FROM FLOPPY CONTROLLER
;
; TRANSFERS ARE SYNCHONIZED BYT MSR D7 <RQM> AND D6 <DIO>
;	RQM  DIO
;	0	0	BUSY
;	1	0	WRITE TO DATA REGISTER PERMITTED
;	1	1	BYTE FOR READ BY HOST PENDING
;	0	1	BUSY
;
;________________________________________________________________________________________________________________________________
;		
GFDATA:
	LDA	>FMSR			; GET STATUS
	BPL	GFDATA			; NOT READY, WAIT
	ANDA	#%01000000		; ANY DATA FOR US?
	BEQ	GFDATA1			; NO, SKIP IT
	LDA	>FDATA			; GET FDC DATA
	ANDCC	#%11111011		; INSURE 'Z' CLEAR
GFDATA1	
	RTS			
		
;__SEGDISPLAY________________________________________________________________________________________
;
;  DISPLAY CONTENTS OF TRACK, SECTOR, ST0, ST1 ON DSKY
;     
;____________________________________________________________________________________________________
SEGDISPLAY:
	PSHS	X
	LDAA	#$FF			;
	STAA	DDRA			;
	STAA	DDRC			;
	LDAA	#$00			;
	STAA	DDRB			;
	LDAA	CYL,U			;
	ANDA	#$0F			;
	STAA	DISPLAYBUF+2		;
	LDAA	CYL,U			;
	ANDA	#$F0			;
	ASRA				;
	ASRA				;
	ASRA				;
	ASRA				;
	STAA	DISPLAYBUF+3		;		
	LDAA	SEC,U			;
	ANDA	#$0F			;
	STAA	DISPLAYBUF		;
	LDAA	SEC,U			;
	ANDA	#$F0			;
	ASRA				;
	ASRA				;
	ASRA				;
	ASRA				;
	STAA	DISPLAYBUF+1		;
	LDAA	HEAD,U			;
	ANDA	#$0F			;
	STAA	DISPLAYBUF+4		;
	LDAA	HEAD,U			;
	ANDA	#$F0			;
	ASRA				;
	ASRA				;
	ASRA				;
	ASRA				;
	STAA	DISPLAYBUF+5		;		
	LDAA	#$01			;
	STAA	DISPLAYBUF+6		;
	LDAA	#$00			;
	STAA	DISPLAYBUF+7		;
	LDX	#DISPLAYBUF		;
	LDAB	#$07	
	ABX
	LDAB	#$08			; SET DIGIT COUNT
	LDAA	#$40			; set Control port 7218 to off
	STAA	PORTC			; output
	JSR 	PAUSE			; wait
	LDAA	#$D0			; set control to 1111 (Data Coming, Hex Decode,NO Decode, Normal)
	STAA	PORTA			; output to port
	LDAA	#$80			; Strobe write pulse with Control=1
	STAA	PORTC			; output to port
	JSR 	PAUSE			; wait
	LDAA	#$40			; set Control port 7218 to off
	STAA	PORTC			; output
SEGDISPLAY_LP:		
	LDAA	,X			; GET DISPLAY DIGIT
	DEX
	STAA	PORTA			; OUT TO PORTA
	LDA	#$00			; SET WRITE STROBE
	STAA	PORTC			; OUT TO PORTC
	JSR	PAUSE			; DELAY
	LDAA	#$40			; SET CONTROL PORT OFF
	STAA	PORTC			; OUT TO PORTC
	JSR	PAUSE			; WAIT
	DECB				; INC POINTER
	CMPB	#$00			;
	BNE	SEGDISPLAY_LP		; LOOP FOR NEXT DIGIT
	PULS	X
	RTS
PAUSE:
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
*	NOP
*	NOP
*	NOP
	RTS

FCMD:	FCB	0			; COMMAND READ OR WRITE,
ST0:	FCB	0			; COMMAND READ OR WRITE,
FLATCH_STORE:
	FCB	00
	
DISPLAYBUF:	FCB 	01,02,03,04,05,06,07,08
;
; DISK COMMAND BLOCK
;
				
		
		
* IGNORE ANY UNUSED INTERRUPTS
IGNORE	RTI
* RESULT CODES FOR FDC OPERATIONS
RESTAB	FCB	3,0,2,0,0,3,4,1
*
* INITIALIZATION TABLE FOR CUBIX RAM
*
RITAB	EQU	*
* DEFAULT DRIVE CHARACTISTICS
	FCB	0,80,2,9,0,0,0		ADR 0, 80 CYL, 2 HEAD, 9 SEC/TRK
	FCB	1,80,2,9,0,0,0		ADR 1, 255 CYL, 1 HEAD, 255 SEC/TRK, ATAPI ZIP DISK
	FCB	2,255,1,255,0,0,0	ADR 2, 255 CYL, 1 HEAD, 255 SEC/TRK
	FCB	3,255,1,255,0,0,0	ADR 4, 255 CYL, 1 HEAD, 255 SEC/TRK
* CONSOLE DEVICE ASSIGNMENTS
	FCB	1		CONSOLE INPUT DEVICE
	FCB	1		CONSOLE OUTPUT DEVICE
* SERIAL DEVICE DRIVERS
	FDB	RDNULL,RDSER1,RDSER2,0,0,0,0,0
	FDB	WRNULL,WRSER1,WRSER2,0,0,0,0,0
* DISK DEVICE DRIVERS
	FDB	DHOME,DRDSEC,DWRSEC,DFORMAT
* 6809 HARDWARE VECTORS
	FDB	SSR		SWI VECTOR (USED FOR SSR'S)
	FDB	IGNORE		SWI2 VECTOR
	FDB	IGNORE		SWI3 VECTOR
	FDB	IGNORE		IRQ  VECTOR
	FDB	IGNORE		FIRQ VECTOR
	FDB	IGNORE		NMI VECTOR
* MISC FLAGS & VARIABLES
	FCB	$FF		ERROR MESSAGES ENABLED
	FCB	0		TRACE DISABLED
	FCB	0		DEBUG DISABLED
	FCB	2		DEFAULT DRIVE (A)
	FCC	'MAIN'		DEFAULT DIRECTORY
	FCB	0,0,0,0		(FILLER)
	FCB	2		SYSTEM DRIVE (A)
	FCC	'SYSTEM'	SYSTEM DIRECTORY
	FCB	0,0		(FILLER)
RISIZ	EQU	*-RITAB		SIZE OF INITILAIZED RAM
